"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const events_1 = tslib_1.__importDefault(require("events"));
const readline_1 = tslib_1.__importDefault(require("readline"));
const logger = require('../logger')('connection');
class Connection extends events_1.default {
    constructor(readable, writeable) {
        super();
        this.readable = readable;
        this.writeable = writeable;
        this._ready = false;
        this.readable = process.stdin;
        this.writeable = process.stdout;
        const rl = readline_1.default.createInterface(this.readable);
        rl.on('line', (line) => {
            this.parseData(line);
        });
        rl.on('close', () => {
            logger.error('stdin get closed');
            process.exit(0);
        });
    }
    parseData(str) {
        if (str.length == 0)
            return;
        let arr;
        try {
            arr = JSON.parse(str);
        }
        catch (e) {
            logger.error('Invalid data from vim', str);
            this.echoerr(`Invalid data: ${str}`);
            return;
        }
        let [id, obj] = arr;
        logger.debug('received request', id, obj);
        if (arr.length > 2) {
            logger.error('Result array length > 2', arr);
        }
        if (id > 0) {
            this.emit('request', id, obj);
        }
        else if (id == 0) {
            if (obj[0] == 'ready') {
                let [channel] = obj[1];
                this._channel = channel;
                this._ready = true;
                this.emit('ready', channel);
            }
            else {
                this.emit('notification', obj);
            }
        }
        else {
            // response for previous request
            this.emit('response', id, obj);
        }
    }
    get channelId() {
        if (this._ready) {
            return Promise.resolve(this._channel);
        }
        return new Promise(resolve => {
            this.once('ready', () => {
                resolve(this._channel);
            });
        });
    }
    get isReady() {
        return this._ready;
    }
    response(requestId, data) {
        this.send([requestId, data || null]);
    }
    notify(event, data) {
        this.send([0, [event, data || null]]);
    }
    send(arr) {
        logger.debug('send response', arr[0], arr.slice(1));
        this.writeable.write(JSON.stringify(arr) + '\n');
    }
    redraw(force = false) {
        this.send(['redraw', force ? 'force' : '']);
    }
    commmand(cmd) {
        this.send(['ex', cmd]);
    }
    echoerr(msg) {
        this.commmand(`echoerr '${msg.replace(/'/, "''")}'`);
    }
    normal(cmd) {
        this.send(['normal', cmd]);
    }
    expr(requestId, expr) {
        if (typeof requestId === 'boolean' && requestId === true) {
            this.send(['expr', expr]);
            return;
        }
        this.send(['expr', expr, requestId]);
    }
    call(requestId, func, args) {
        if (typeof requestId === 'boolean' && requestId === true) {
            // notify
            this.send(['call', func, args]);
            return;
        }
        this.send(['call', func, args, requestId]);
    }
}
exports.default = Connection;
//# sourceMappingURL=connection.js.map